
import 'dart:async';
import 'dart:html' as html;
import 'dart:js' as js;
import 'dart:js';
import 'dart:ui_web';
import "dart:convert";
import 'dart:ui_web' as ui_web;
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter/services.dart';
import 'package:flutter_web_plugins/flutter_web_plugins.dart';
import 'enx_flutter_plugin.dart';


class JavaScriptUpdate {
  final String methodName;
  final dynamic data;

  JavaScriptUpdate(this.methodName, this.data);
}

/// A web implementation of the EnxFlutterPluginPlatform of the EnxFlutterPlugin plugin.
class EnxFlutterPluginWeb {

  EnxFlutterPluginWeb(){
    injectJSLibraries();
    setupJavaScriptCallback();
    registerJsCallback();
  }

 static late EventChannel eventChannel;
  static late MethodChannel methodChannel;
 static late var jsData1;
  static Map<int, html.Element> viewMap = {};
  static Map<int, html.Element> registeredViews = {};
  static void registerWith(Registrar registrar) {
   methodChannel = MethodChannel(
      'enx_flutter_plugin',
      const StandardMethodCodec(),
      registrar,
    );

   if(kIsWeb){

     ui_web.platformViewRegistry.registerViewFactory("EnxPlayer", (int viewId, {Object? params}) {
       // Get the creationParams passed from Flutter
       // Get the creationParams passed from Flutter
       Map<String, dynamic> jsonMap = jsonDecode(jsonEncode(params).toString());
       int uid = jsonMap['setZOrderMediaOverlay']['uid'];
       print('enxPlayeriD $uid');

       // Check if the view is already registered
       if (registeredViews.containsKey(uid)) {
         // If already registered, return the existing HTML element
         return registeredViews[uid]!;
       } else {
         // Create a new HTML element
         final html.Element htmlElement = html.DivElement()
           ..id = '$uid'
           ..style.width = '100%'
           ..style.height = '100%';

         // Add the element to the document body
         html.document.body!.append(htmlElement);

         // Store the element in the map
         registeredViews[uid] = htmlElement;

         // Call JavaScript method to create native view
         js.context.callMethod("createNativeView", [uid]);

         // Return an object representing the platform view (in this case, return the HTML element)
         return htmlElement;
       }
     });
    final EnxFlutterPluginWeb instance = EnxFlutterPluginWeb();
    methodChannel.setMethodCallHandler(instance.handleMethodCall);

}
  }
  Future<void> injectJSLibraries() async {
  if(kIsWeb){
  final List<Future<void>> loading = <Future<void>>[];
  final List<html.HtmlElement> tags = <html.HtmlElement>[];

  final enxJsUrl = assetManager.getAssetUrl('packages/enx_flutter_plugin/assets/EnxRtc.js');
  final html.ScriptElement enxScript = html.ScriptElement()
    ..async = true
    ..defer = true
    ..src = enxJsUrl;
  loading.add(enxScript.onLoad.first);
  tags.add(enxScript);
  html.querySelector('head')!.children.addAll(tags);
  // ignore: undefined_prefixed_name
  final jsUrl1 = assetManager.getAssetUrl('packages/enx_flutter_plugin/assets/enx_flutter_plugin.js');
  final html.ScriptElement script = html.ScriptElement()
    ..async = true
    ..defer = true
    ..src = jsUrl1;
  loading.add(script.onLoad.first);
  tags.add(script);
  html.querySelector('head')!.children.addAll(tags);
  await Future.wait(loading);

  final completer = Completer<void>();
  script.addEventListener('load', (event) => {completer.complete()});
  return completer.future;

}

  }
  // Define a callback function to receive data from JavaScript
  Completer<List<dynamic>> _jsCallbackCompleter = Completer<List<dynamic>>();

  // Register a Dart function as a callback in JavaScript
  void registerJsCallback() {
    js.context['getDevicesCallback'] = js.allowInterop((data) {
      // Convert JavaScript array to Dart list
      List<dynamic> devices = json.decode(data);

      // Resolve the completer with the received data
      _jsCallbackCompleter.complete(devices);
    });
  }
  // Method channel method handler
  Future<dynamic> handleMethodCall(MethodCall call) async {
    if(kIsWeb){

      switch (call.method) {
        case 'joinRoom':

          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('joinRoomJs', [jsData]);

          break;
        case 'setupVideo':
           jsData1 = JsObject.jsify(call.arguments);
          // js.context.callMethod("createNativeView",[jsData1]);

          break;
        case 'publish':
          break;
        case 'disconnect':
          js.context.callMethod('disConnectRoom');
          break;
        case 'switchMediaDevice':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('switchMediaDevice', [jsData]);

          break;

        case 'muteSelfAudio':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('muteSelfAudio', [jsData]);
          break;
        case 'muteSelfVideo':
          var jsData = JsObject.jsify(call.arguments);

          js.context.callMethod('muteSelfVideo', [jsData]);
          break;
        case 'switchCamera':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('switchCamera', [jsData]);

          break;
        case 'getDevices':
        // Reset the Completer each time
          _jsCallbackCompleter = Completer<List<dynamic>>();
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('getAllDevicesList', [jsData]);
         return _jsCallbackCompleter.future;
        case 'destroy':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('destroy', [jsData]);
          break;
        case 'extendConferenceDuration':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('extendConferenceDuration', [jsData]);
          break;
        case 'lockRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('lockRoom', [jsData]);
          break;
        case 'unLockRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('unlockRoom', [jsData]);
          break;

        case 'startRecord':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('startRecord', [jsData]);

          break;
        case 'stopRecord':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('stopRecord', [jsData]);
          break;

        case 'setTalkerCount':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('setTalkerCount', [jsData]);
          break;

        case 'getTalkerCount':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('getTalkerCount', [jsData]);

          break;
        case 'getMaxTalkers':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('getMaxTalkers', [jsData]);

          break;
        case 'hardMute':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardMute', [jsData]);
          break;

        case 'hardUnMute':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardUnMute', [jsData]);
          break;

        case 'hardMuteAudio':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardUnMute', [jsData]);
          break;
        case 'hardUnMuteAudio':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardUnMute', [jsData]);
          break;
        case 'hardMuteVideo':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardUnMute', [jsData]);
          break;
        case 'hardUnMuteVideo':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('hardUnMute', [jsData]);
          break;
        case 'approveAwaitedUser':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('approveAwaitedUser', [jsData]);
          break;
        case 'denyAwaitedUser':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('denyAwaitedUser', [jsData]);
          break;

        case 'switchUserRole':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('switchUserRole', [jsData]);
          break;
        case 'pinUsers':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('pinUsers', [jsData]);
          break;
        case 'unpinUsers':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('unpinUsers', [jsData]);
          break;
        case 'addSpotlightUsers':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('addSpotlightUsers', [jsData]);
          break;
        case 'removeSpotlightUsers':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('removeSpotlightUsers', [jsData]);
          break;
        case 'switchRoomMode':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('switchRoomMode', [jsData]);
          break;
        case 'sendMessage':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('sendMessage', [jsData]);
          break;
        case 'sendUserData':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('sendUserData', [jsData]);
          break;
        case 'sendFiles':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('sendFiles', [jsData]);
          break;
        case 'startScreenShare':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('startScreenShare', [jsData]);
          break;
        case 'stopScreenShare':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('stopScreenShare', [jsData]);
          break;
        case 'startStreaming':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('startStreaming', [jsData]);
         break;
        case 'stopStreaming':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('stopStreaming', [jsData]);
          break;
        case 'startLiveRecording':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('startLiveRecording', [jsData]);
          break;
        case 'stopLiveRecording':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('stopLiveRecording', [jsData]);
          break;
        case 'startLiveTranscriptionForRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('startLiveTranscriptionForRoom', [jsData]);
          break;
        case 'stopLiveTranscription':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('stopLiveTranscription', [jsData]);
          break;
        case 'subscribeForTalkerNotification':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('subscribeForTalkerNotification', [jsData]);
          break;
        case 'enableStats':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('enableStats', [jsData]);
          break;
        case 'setAudioOnlyMode':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('setAudioOnlyMode', [jsData]);
          break;
        case 'makeOutboundCall':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('makeOutboundCall', [jsData]);
          break;
        case 'cancelOutboundCall':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('cancelOutboundCall', [jsData]);
          break;
        case 'sendDTMF':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('sendDTMF', [jsData]);
          break;
        case 'captureScreenShot':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('captureScreenShot', [jsData]);
          break;

        case 'requestFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('requestFloor', [jsData]);
          break;
        case 'grantFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('grantFloor', [jsData]);
          break;
        case 'cancelFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('cancelFloor', [jsData]);
          break;
        case 'finishFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('finishFloor', [jsData]);
          break;
        case 'denyFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('denyFloor', [jsData]);
          break;
        case 'releaseFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('releaseFloor', [jsData]);
          break;

        case 'inviteToFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('inviteToFloor', [jsData]);
          break;
        case 'cancelFloorInvite':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('cancelFloorInvite', [jsData]);
          break;
        case 'rejectInviteFloor':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('rejectInviteFloor', [jsData]);
          break;
        case 'acceptInviteFloorRequest':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('acceptInviteFloorRequest', [jsData]);
          break;
        case 'createBreakOutRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('createBreakOutRoom', [jsData]);
          break;
        case 'createAndInviteBreakoutRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('createAndInviteBreakoutRoom', [jsData]);
          break;
        case 'joinBreakOutRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('joinBreakOutRoom', [jsData]);
          break;
        case 'inviteToBreakOutRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('inviteToBreakOutRoom', [jsData]);
          break;
        case 'pause':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('pause', [jsData]);
          break;
        case 'resume':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('resume', [jsData]);
          break;
        case 'muteRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('muteRoom', [jsData]);
          break;
        case 'unmuteRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('unmuteRoom', [jsData]);
          break;
        case 'rejectBreakOutRoom':
          var jsData = JsObject.jsify(call.arguments);
          js.context.callMethod('rejectBreakOutRoom', [jsData]);
          break;
        default:
          throw PlatformException(
            code: 'Unimplemented',
            details: 'Method ${call.method} not implemented on web',
          );
      }
    }

  }

  // Event channel event handler
  void handleEvent(dynamic event) {
      // Implement event handling for web
     sendDataToPlugin(event) ;





  }
  void setupJavaScriptCallback() {
    if(kIsWeb){
      js.context['dartCallback'] = js.allowInterop((data) {
        handleJavaScriptUpdate(JavaScriptUpdate(data['methodName'], data['data']));
      });
    }


    }
  void handleJavaScriptUpdate(JavaScriptUpdate update) {
    print("jsonData ${update.data}");
    dynamic dartMap = jsonDecode(update.data);
    Map<String, dynamic> data={};
    if (dartMap is List<dynamic>) {
      List<Map<String, dynamic>> listOfMaps = [];
      for (var item in dartMap) {
        if (item is Map<String, dynamic>) {
          listOfMaps.add(item);
        }
      }
      data['activeList'] = listOfMaps;
     // data.addEntries({"activeList":listOfMaps}.entries);
    }else{
      data=dartMap;
    }

    print('Received update from JavaScript - Method: ${update.methodName}, Data: ${update.data}');
    switch (update.methodName) {
      case 'room-connected':
       // print('Method room-connected ${update.data.toString()}');
        data.addEntries({"event":"onRoomConnected"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-error':
        print('room-error ${update.data.toString()}');
        data.addEntries({"event":"onRoomError"}.entries);
        EnxRtc.receiveDataFromWeb(data);

        break;
       case 'room-allowed':
         print('room-allowed ${update.data.toString()}');

        break;
      case 'user-connected':
        print('user-connected ${update.data.toString()}');
        data.addEntries({"event":"onUserConnected"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-awaited':
        print('user-awaited ${update.data.toString()}');


        break;
      case 'active-talkers-updated':
        data.addEntries({"event":"onActiveTalkerList"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        print('active-talkers-updated ${update.data.toString()}');

        break;
      case 'onAudioEvent':
        data.addEntries({"event":"onAudioEvent"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-audio-muted':
        data.addEntries({"event":"onRemoteStreamAudioMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);

        data.addEntries({"event":"onReceivedHardMuteAudio"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-audio-unmuted':
        print('user-audio-unmuted ${update.data.toString()}');
        data.addEntries({"event":"onRemoteStreamAudioUnMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);

        data.addEntries({"event":"onReceivedHardUnMuteAudio"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onVideoEvent':
        print('user-video-muted ${update.data.toString()}');
        data.addEntries({"event":"onVideoEvent"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-video-muted':
        print('user-video-muted ${update.data.toString()}');
        data.addEntries({"event":"onRemoteStreamVideoMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        data.addEntries({"event":"onReceivedHardMuteVideo"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-video-unmuted':
        data.addEntries({"event":"onRemoteStreamVideoUnMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);

        data.addEntries({"event":"onReceivedHardUnMuteVideo"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onHardMuted':
        data.addEntries({"event":"onHardMuted"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onHardUnMuted':
        data.addEntries({"event":"onHardUnMuted"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hard-mute-room':
        data.addEntries({"event":"onReceivedHardMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hard-unmute-room':
        data.addEntries({"event":"onReceivedHardUnMute"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'hardmute-user-audio':
        data.addEntries({"event":"onAckHardMuteUserAudio"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hardunmute-user-audio ':
        data.addEntries({"event":"onAckHardunMuteUserAudio"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hardmute-user-video':
        data.addEntries({"event":"onAckHardMuteUserVideo"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hardunmute-user-video':
        data.addEntries({"event":"onAckHardUnMuteUserVideo"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'room-disconnected':
        print('room-disconnected ${update.data.toString()}');
        dartMap.addEntries({"event":"onRoomDisConnected"}.entries);

        EnxRtc.receiveDataFromWeb(dartMap);

        break;
      case 'user-disconnected':
        print('user-disconnected ${update.data.toString()}');
        data.addEntries({"event":"onUserDisConnected"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'removeView':
        removeView(data['id']);

        break;
      case 'getDevices':
        data.addEntries({"event":"getDevices"}.entries);
        EnxRtc.receiveDataFromWeb(data);
      case 'stream-added':
        data.addEntries({"event":"onStreamAdded"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'stream-subscribed':
        data.addEntries({"event":"onSubscribedStream"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-locked':
        data.addEntries({"event":"onLockedRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-unlocked':
        data.addEntries({"event":"onUnLockedRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'conference-remaining-duration':
        data.addEntries({"event":"onConferenceRemainingDuration"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onConferencessExtended':
        data.addEntries({"event":"onConferencessExtended"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onSetTalkerCount':
        data.addEntries({"event":"onSetTalkerCount"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onGetTalkerCount':
        data.addEntries({"event":"onGetTalkerCount"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onMaxTalkerCount':
        data.addEntries({"event":"onMaxTalkerCount"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-record-on':
        data.addEntries({"event":"onRoomRecordingOn"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-record-off':
        data.addEntries({"event":"onRoomRecordingOff"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onStartRecordingEvent':
        data.addEntries({"event":"onStartRecordingEvent"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onStopRecordingEvent':
        data.addEntries({"event":"onStopRecordingEvent"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onAckForApproveAwaitedUser':
        data.addEntries({"event":"onAckForApproveAwaitedUser"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckForDenyAwaitedUser':
        data.addEntries({"event":"onAckForDenyAwaitedUser"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-role-changed':
        data.addEntries({"event":"onUserRoleChanged"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onSwitchedUserRole':
        data.addEntries({"event":"onSwitchedUserRole"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckPinUsers':
        data.addEntries({"event":"onAckPinUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckUnpinUsers':
        data.addEntries({"event":"onAckUnpinUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'updated-pinned-users':
        data.addEntries({"event":"onPinnedUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onAckAddSpotlightUsers':
        data.addEntries({"event":"onAckAddSpotlightUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckRemoveSpotlightUsers':
        data.addEntries({"event":"onAckRemoveSpotlightUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'spotlight-users':
        data.addEntries({"event":"onUpdateSpotlightUsers"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckSwitchedRoom':
        data.addEntries({"event":"onAckSwitchedRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-mode-switched':
        data.addEntries({"event":"onRoomModeSwitched"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'message-received':
        data.addEntries({"event":"onMessageReceived"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-data-received':
        data.addEntries({"event":"onUserDataReceived"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAcknowledgedSendData':
        data.addEntries({"event":"onAcknowledgedSendData"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'share-started':
        data.addEntries({"event":"onScreenSharedStarted"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'share-stopped':
        data.addEntries({"event":"onScreenSharedStopped"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onStartScreenShareACK':
        data.addEntries({"event":"onStartScreenShareACK"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onStoppedScreenShareACK':
        data.addEntries({"event":"onStoppedScreenShareACK"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckStartStreaming':
        data.addEntries({"event":"onAckStartStreaming"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckStopStreaming':
        data.addEntries({"event":"onAckStopStreaming"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'streaming-started':
        data.addEntries({"event":"onStreamingStarted"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'streaming-stopped':
        data.addEntries({"event":"onStreamingStopped"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'streaming-failed':
        data.addEntries({"event":"onStreamingFailed"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'streaming-updated':
        data.addEntries({"event":"onStreamingUpdated"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onACKStartLiveRecording':
        data.addEntries({"event":"onACKStartLiveRecording"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onACKStopLiveRecording':
        data.addEntries({"event":"onACKStopLiveRecording"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-live-recording-on':
        data.addEntries({"event":"onRoomLiveRecordingOn"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-live-recording-failed':
        data.addEntries({"event":"onRoomLiveRecordingFailed"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-live-recording-updated':
        data.addEntries({"event":"onRoomLiveRecordingUpdate"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-live-recording-off':
        data.addEntries({"event":"onRoomLiveRecordingOff"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onACKStartLiveTranscription':
        data.addEntries({"event":"onACKStartLiveTranscription"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onACKStopLiveTranscription':
        data.addEntries({"event":"onACKStopLiveTranscription"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'self-transcription-on':
        data.addEntries({"event":"onSelfTranscriptionOn"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-transcription-on':
        data.addEntries({"event":"onRoomTranscriptionOn"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'transcription-events':
        data.addEntries({"event":"onTranscriptionEvents"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'self-transcription-off':
        data.addEntries({"event":"onSelfTranscriptionOff"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'room-transcription-off':
        data.addEntries({"event":"onRoomTranscriptionOff"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hls_started':
        data.addEntries({"event":"onHlsStarted"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hls_stopped':
        data.addEntries({"event":"onHlsStopped"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hls_failed':
        data.addEntries({"event":"onHlsFailed"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'hls_waiting':
        data.addEntries({"event":"onHlsWaiting"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckSubscribeTalkerNotification':
        data.addEntries({"event":"onAckSubscribeTalkerNotification"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckUnsubscribeTalkerNotification':
        data.addEntries({"event":"onAckUnsubscribeTalkerNotification"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'talker-notification':
        data.addEntries({"event":"onTalkerNotification"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onOutBoundCallInitiated':
        data.addEntries({"event":"onOutBoundCallInitiated"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'dial-state-events':
        data.addEntries({"event":"onDialStateEvents"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onOutBoundCallCancel':
        data.addEntries({"event":"onOutBoundCallCancel"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onOutBoundCallSendDtmf':
        data.addEntries({"event":"onOutBoundCallSendDtmf"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onProcessFloorRequested':
        data.addEntries({"event":"onProcessFloorRequested"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

      case 'onFloorRequested':
        data.addEntries({"event":"onFloorRequested"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onFloorCancelled':
        data.addEntries({"event":"onFloorCancelled"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onFloorFinished':
        data.addEntries({"event":"onFloorFinished"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onACKInviteToFloorRequested':
        data.addEntries({"event":"onACKInviteToFloorRequested"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-requested':
        data.addEntries({"event":"onFloorRequestReceived"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-cancelled':
        data.addEntries({"event":"onCancelledFloorRequest"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-denied':
        data.addEntries({"event":"onDeniedFloorRequest"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-granted':
        data.addEntries({"event":"onGrantedFloor"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-finished':
        data.addEntries({"event":"onFinishedFloorRequest"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'release-floor':
        data.addEntries({"event":"onReleasedFloorRequest"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-invited':
        data.addEntries({"event":"onInviteToFloorRequested"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        data.addEntries({"event":"onInvitedForFloorAccess"}.entries);
        EnxRtc.receiveDataFromWeb(data);

        break;
      case 'floor-invite-cancelled':
        data.addEntries({"event":"onCanceledFloorInvite"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-accepted':
        data.addEntries({"event":"onAcceptedFloorInvite"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'floor-rejected':
        data.addEntries({"event":"onRejectedInviteFloor"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckCreateBreakOutRoom':
        data.addEntries({"event":"onAckCreateBreakOutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckCreateAndInviteBreakOutRoom':
        data.addEntries({"event":"onAckCreateAndInviteBreakOutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckInviteBreakOutRoom':
        data.addEntries({"event":"onAckInviteBreakOutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckPause':
        data.addEntries({"event":"onAckPause"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckResume':
        data.addEntries({"event":"onAckResume"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckMuteRoom':
        data.addEntries({"event":"onAckMuteRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckUnmuteRoom':
        data.addEntries({"event":"onAckUnmuteRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'onAckRejectBreakOutRoom':
        data.addEntries({"event":"onAckRejectBreakOutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'join-breakout-room':
        data.addEntries({"event":"onUserJoinedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-room-joining':
        data.addEntries({"event":"onBreakoutroomjoining"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-room-connected':
        data.addEntries({"event":"onConnectedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-room-error':
        data.addEntries({"event":"onFailedJoinBreakOutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-joined-breakout-room':
        data.addEntries({"event":"onUserJoinedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-invite-rejected':
        data.addEntries({"event":"onBreakoutRoomInviteRejected"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-room-disconnected':
        data.addEntries({"event":"onDisconnectedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'user-disconnected-breakout-room':
        data.addEntries({"event":"onUserDisconnectedFromBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-user-disconnected':
        data.addEntries({"event":"onDisconnectedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;
      case 'breakout-room-destroyed':
        data.addEntries({"event":"onDisconnectedBreakoutRoom"}.entries);
        EnxRtc.receiveDataFromWeb(data);
        break;

    }
   }
  static void sendDataToPlugin(Map<String, dynamic> map) async{
    print('new dat js ');
    try {
      await methodChannel.invokeMethod('receiveDataFromWeb', map);
    } on PlatformException catch (e) {
      print("Failed to send data to plugin: '${e.message}'.");
    }
  }

// Example function to update a registered view
  void updateView(int viewId, String content) {
    if (registeredViews.containsKey(viewId)) {
      // Retrieve the HTML element
      final html.Element viewElement = registeredViews[viewId]!;

      // Update the content of the view
      viewElement.innerHtml = content;
    }
  }
  // Example function to remove a registered view
  void removeView(int viewId) {
    if (registeredViews.containsKey(viewId)) {
      // Retrieve the HTML element
      final html.Element viewElement = registeredViews[viewId]!;

      // Remove the view from the document body
      viewElement.remove();

      // Remove the view from the map
      registeredViews.remove(viewId);
    }
  }
}
